<?php
namespace PME\Controllers\Medicao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use \Estoque\Entities\Lanctoestoq;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;

class RetiradaInventarioController {
	protected $environment;
	protected $connection;
  protected $lista;
  protected $newCode;
  protected $estoqueService;

  public function __construct($connection, $environment, $lista, $newCode, ValidacoesEstoqueService $estoqueService, $entityManager) {
    $this->connection  = $connection;
    $this->environment = $environment;
    $this->lista       = $lista;
    $this->newCode     = $newCode;
    $this->estoqueService = $estoqueService;
    $this->entityManager  = $entityManager;
    
    \Setforms2\Shared\Registry::store("EntityManager", $entityManager);
    // \Setforms2\Shared\Registry::store("EntityManagerConnection", $entityManager->getConnection());
	}
  
  public function configRetirada(DTO\Request $request, DTO\Response $response) {
    try {
      $params = $request->getParameters();
      $data = $this->getParamplaneja($params['CDCENTRAL']);
      $params['CDFILIAL'] = $params['CDCENTRAL'];
      $paramfilial = $this->getParamFilial($params);
      
      $response->addDataSet(new DataSet("result", array('paramplaneja'=> $data, 'paramfilial'=> $paramfilial)));
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }


  private function getParamplaneja($unidade) {
    $query = "SELECT P.IDPRINVENRETI, P.CDSERVRETPME, S.NMSERVICO, P.CDTPRETIPADR, S.CDSERVFILI, S.IDSERVATIV, PF.IDUTRETBASEINVE
                FROM PARAMPLANEJA P
                LEFT JOIN SERVICO S ON S.CDFILIAL = P.CDFILIAL AND S.CDSERVFILI = P.CDSERVRETPME
                LEFT JOIN PARAMFILIAL PF ON P.CDFILIAL = PF.CDFILIAL
              WHERE P.CDFILIAL = :P_CDFILIAL";
    $params = array (
      "P_CDFILIAL" => $unidade
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }
  
  public function returnRowsCliente(&$rows){
    try{
      $query = "SELECT SERVCLIEPME.CDFILIAL, SERVCLIEPME.CDSERVFILI, SERVCLIEPME.CDFILCENTPME, SERVCLIEPME.CDTIPOCARD, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTEFETCLPME, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTCARDIAPME, SERVCLIEPME.CDCLIENTE, CLIENTE.NMRAZSOCCLIE, EFETCLIEPME.QTCOMEFCLPME, EFETCLIEPME.QTFUNEFCLPME, EFETCLIEPME.QTDOAEFCLPME, SERVCLIEPME.NRORG FROM CLIENTE, SERVCLIEPME LEFT JOIN EFETCLIEPME ON EFETCLIEPME.DTEFETCLPME = :DTEFETCLIE AND SERVCLIEPME.CDFILCENTPME = EFETCLIEPME.CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = EFETCLIEPME.CDTIPOCARD AND SERVCLIEPME.CDFILIAL = EFETCLIEPME.CDFILIAL AND SERVCLIEPME.CDSERVFILI = EFETCLIEPME.CDSERVFILI AND SERVCLIEPME.CDCLIENTE = EFETCLIEPME.CDCLIENTE WHERE SERVCLIEPME.CDFILCENTPME = :CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = :CDTIPOCARD AND SERVCLIEPME.CDFILIAL = :CDFILIAL AND SERVCLIEPME.CDSERVFILI = :CDSERVFILI AND SERVCLIEPME.CDCLIENTE = CLIENTE.CDCLIENTE ORDER BY CLIENTE.NMRAZSOCCLIE";
      $params = array (
        "DTEFETCLIE" => $rows[0]['DTEFETCLIE'],
        "CDFILCENTPME" => $rows[0]['CDFILCENTPME'],
        "CDTIPOCARD" => $rows[0]['CDTIPOCARD'],
        "CDFILIAL" => $rows[0]['CDFILIAL'],
        "CDSERVFILI" => $rows[0]['CDSERVFILI']
      );
      $data = $this->connection->fetchAll($query, $params);
      foreach ($data as $row){
        $valid = true;
        foreach ($rows as $editable) {
          if ($editable['CDCLIENTE'] == $row['CDCLIENTE']) {
            $valid = false;  
          }  
        }
        if ($valid) {
          array_push($rows, $row);
        }
      }
    } catch (\Exception $e) {
	    throw $e;
	  }
  }

  public function saveItemsRomaneio(DTO\Request $request, DTO\Response $response) {
    try{
      $rows = $request->getRow();
      $nrSeq = $this->getMaxSeqItem($rows[0]['CDFILIAL'], $rows[0]['NRINVENTRETI']) + 1;
      
      $this->connection->beginTransaction();

      $queryInsert = "INSERT INTO ITEMINVENTRETI (CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV)
                      VALUES (:CDFILIAL, :NRINVENTRETI, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV)";

      $queryUpdate = "UPDATE ITEMINVENTRETI SET QTITINVENTRETI = :QTITINVENTRETI
                      WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI AND NRITINVENTRETI = :NRITINVENTRETI 
                      AND CDPRODUTO = :CDPRODUTO AND IDORIGLANCPRINV = :IDORIGLANCPRINV";

      foreach($rows as $row) {
        if($row['__is_new']){
          $row['IDORIGLANCPRINV'] = 0;
          $row['NRITINVENTRETI'] = $nrSeq;
          $nrSeq += 1;
          $stmt = $this->connection->prepare($queryInsert);
        } else {
          $stmt = $this->connection->prepare($queryUpdate);
        } 

        $params = array(
          'CDFILIAL' => $row['CDFILIAL'],
          'NRINVENTRETI' => $row['NRINVENTRETI'],
          'NRITINVENTRETI' => $row['NRITINVENTRETI'],
          'QTITINVENTRETI' => $row['QTITINVENTRETI'],
          'CDPRODUTO' => $row['CDPRODUTO'],
          'IDORIGLANCPRINV' => $row['IDORIGLANCPRINV']
        );

        /* if($row['__is_new']) $stmt = $this->connection->prepare($queryInsert);
        else $stmt = $this->connection->prepare($queryUpdate);*/
        
        $stmt->execute($params);
      }
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function deleteItemRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $rows = $request->getRow();
      $this->connection->beginTransaction();

      $this->deleteRomaneioEItem($rows[0]);

      $this->connection->commit();
      $response->addDataSet(new DataSet('response', ['true']));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function deleteRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $rows = $request->getRow();
      $this->connection->beginTransaction();

      foreach($rows as $row){
        $this->deleteRomaneioEItem($row);
      }

      $this->connection->commit();
      $response->addDataSet(new DataSet('response', ['true']));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function deleteRomaneioEItem($row){
    $queryItems = "DELETE FROM ITEMINVENTRETI WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI";
    $queryRomaneio = "DELETE FROM INVENTARIORETI WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI";
    $params = array(
      "CDFILIAL" => $row["CDFILIAL"],
      "NRINVENTRETI" => $row["NRINVENTRETI"]
    );

    $this->connection->prepare($queryItems)->execute($params);
    $this->connection->prepare($queryRomaneio)->execute($params);
  }

  private function getMaxSeqItem($cdfilial, $nrinvent) {
    $query = "SELECT NVL(MAX(TO_NUMBER(NRITINVENTRETI)), 0) MAXITINVENT FROM ITEMINVENTRETI WHERE CDFILIAL = :P_CDFILIAL AND NRINVENTRETI = :P_NRINVENTRETI";
		$params = array(
			"P_CDFILIAL" => $cdfilial,
			"P_NRINVENTRETI" 	=> $nrinvent
		);
		$stmt = $this->connection->prepare($query);
    $stmt->execute($params);
    $data = $stmt->fetchAll();
    return $data[0]['MAXITINVENT'];
  }

  private function getMaxSeqRomaneio($cdfilial) {
    $query = "SELECT LPAD(TO_CHAR(NVL((MAX(TO_NUMBER(NRINVENTRETI)) + 1) , 1)), 10, 0 ) AS NRINVENTRETI FROM INVENTARIORETI WHERE CDFILIAL = :P_CDFILIAL";
		$params = array(
			"P_CDFILIAL" => $cdfilial
		);
		$stmt = $this->connection->prepare($query);
    $stmt->execute($params);
    $data = $stmt->fetchAll();
    return $data[0]['NRINVENTRETI'];
  }

  public function saveRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $row = $request->getParameters();

      $romaneio = $row['romaneio'];
      $item = $row['item'];

      $this->connection->beginTransaction();
      
      //insert Romaneio
      if(empty($item['NRINVENTRETI'])){
        $nrSeqRom = $this->getMaxSeqRomaneio($romaneio['CDFILIAL']);
        $queryRomaneio = "INSERT INTO INVENTARIORETI (CDFILIAL, NRINVENTRETI, DTINVENTRETI, DSINVENTRETI, IDRETIEFET)
                          VALUES (:CDFILIAL, :NRINVENTRETI, :DTINVENTRETI, :DSINVENTRETI, :IDRETIEFET)";
        $params = array(
          'CDFILIAL' => $romaneio['CDFILIAL'],
          'NRINVENTRETI' => $nrSeqRom,
          'DTINVENTRETI' => $romaneio['DTINVENTRETI'],
          'DSINVENTRETI' => $romaneio['DSINVENTRETI'],
          'IDRETIEFET'   => 'N'
        );
        $stmt = $this->connection->prepare($queryRomaneio);   
        $stmt->execute($params);
        $item['NRINVENTRETI'] = $nrSeqRom;
        
        $nrSeqItem = 1;
      }else $nrSeqItem = $this->getMaxSeqItem($item['CDFILIAL'], $item['NRINVENTRETI']) + 1;
                         
      //insert Items
      $queryItem = "INSERT INTO ITEMINVENTRETI (CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV)
                    VALUES (:CDFILIAL, :NRINVENTRETI, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV)";

      $paramsItem = array(
        'CDFILIAL' => $romaneio['CDFILIAL'],
        'NRINVENTRETI' => $item['NRINVENTRETI'],
        'NRITINVENTRETI' => $nrSeqItem,
        'QTITINVENTRETI' => $item['QTITINVENTRETI'],
        'CDPRODUTO' => $item['CDPRODUTO'],
        'IDORIGLANCPRINV' => 0
      );

      $stmtItem = $this->connection->prepare($queryItem);   
      $stmtItem->execute($paramsItem);
      // $nrSeqItem += 1;

      $this->connection->commit();
      $response->addDataSet(new DataSet('nrseqinvent', [$item['NRINVENTRETI']]));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  // Retirada Automática
  public function validateAutoRemoval(DTO\Request $request, DTO\Response $response){
    try{
      $row = $request->getParameters();

      // $this->prodInactiveOrStock($row);
      
      $row['DTVALESTOQ'] = $row['DATAINVENT'] = $row['DTINVENTRETI'];
      $row['DTINICIAL'] = $this->getIniDate($row);
      $response->addDataSet(new DataSet('params', [$row]));
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function getIniDate($row) {
    $dtIni = '';
    $query = "SELECT TO_CHAR(MAX(DTINVENTRETI), 'DD/MM/YYYY') AS DTMAXINVENTMERE
              FROM INVENTARIORETI
            WHERE CDFILIAL = :P_CDFILIAL
              AND DTINVENTRETI <= :P_DTFINAL
              AND NRINVENTRETI <> :P_NRINVENTRETI";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_DTFINAL' => $row['DTINVENTRETI'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
    );
    $data = $this->connection->fetchAll($query, $params);

    if(!empty($data)) $dtIni = $data[0]['DTMAXINVENTMERE'];
    else {
      $data = $this->getLastRelease($row);
      if(!empty($data)) $dtIni = $data[0]['DTLANCESTQ'];
      else throw new \Exception('Data do último inventário não encontrada.');
    }
    return $dtIni;
  }

  private function getLastRelease($row) {
    $query = "SELECT TO_CHAR(MAX(DTLANCESTQ), 'DD/MM/YYYY') AS DTLANCESTQ
                FROM LANCTOESTOQ
              WHERE CDFILIMOVI = :P_CDFILIAL
                AND DTLANCESTQ <= :P_DTFINAL
                AND IDTPLANCTO IN('0', '1')";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_DTFINAL' => $row['DTINVENTRETI'],
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }
  
  private function prodInactiveOrStock($row) {
    $query = "SELECT P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.SGUNIDADE, F.CDFILIAL, NVL(F.IDCNTRESTOQ, 'N') IDCNTRESTOQ, 
                    NVL(F.IDPRODATIV, 'N') IDPRODATIV
               FROM PARAMPLANEJA PARAM 
              INNER JOIN PRODUTO P ON P.CDPRODUTO = P.CDPRODESTO
              LEFT JOIN PRODFILI F ON F.CDFILIAL = :P_CDFILIAL AND F.CDPRODUTO = P.CDPRODUTO
              INNER JOIN ITEMINVENTRETI IT ON IT.CDFILIAL = :P_CDFILIAL AND IT.NRINVENTRETI = :P_NRINVENTRETI AND IT.CDPRODUTO = P.CDPRODUTO
            WHERE PARAM.CDFILIAL = :P_CDFILIAL
                AND (IDCNTRESTOQ = 'N' OR IDPRODATIV = 'I')
            ORDER BY P.CDARVPROD";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
    );

    $data = $this->connection->fetchAll($query, $params);

    if(count($data) > 0){
      $cdlista = '';
      foreach($data as $item) {
        $cdlista .= $item['CDPRODUTO'].'_';
      }
      $nrlista = $this->lista->geraLista($cdlista, $this->environment->getNrOrg());

      if($data[0]['IDPRODATIV'] == 'I') throw new \Exception('Operação bloqueada! Produto não cadastrado para unidade ou inativo._'.$nrlista);
      else throw new \Exception('Produtos que não controlam estoque_'.$nrlista);
    } 
  }

  public function findParamFilial(DTO\Request $request, DTO\Response $response){
    try {
      $row = $request->getRow();
      $result  = $this->getParamFilial($row);
      $response->addDataSet(new DataSet('paramfilial', [$result]));
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function autoRemoval(DTO\Request $request, DTO\Response $response){
    // atributo da conexão necessário para commitar lista de produtos irregulares depois do rollback
    $this->connection->setNestTransactionsWithSavepoints(true);
    
    $this->connection->beginTransaction('connection');
    try{
      $row = $request->getParameters();
      // $paramfilial = $this->getParamFilial($row);
      $paramfilial = $row['paramfilial'];
      
      $nrorg = $this->environment->getNrOrg();
      
      $date = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
      $dateInvent = \DateTime::createFromFormat('d/m/Y', $row['DATAINVENT']);
      if (!$date) $date = $dateInvent;
      
      $dateFin = \DateTime::createFromFormat('d/m/Y', $row['DTINVENTRETI']);
      $dateEstoq = \DateTime::createFromFormat('d/m/Y', $row['DTVALESTOQ']);
      $dateEstoq->modify('+1 day');
      $row['DTVALESTOQ'] = $dateEstoq->format('d/m/Y');
      
      if($date > $dateFin) {
        $row['DTINICIAL'] = $row['DTINVENTRETI'];
      }
      
      $paramPlaneja = $row['paramplaneja'];
      if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('validatedParamPlaneja', $row) || $row['validatedParamPlaneja'] === false) {
        $paramPlan = $this->validateParamplaneja($paramPlaneja, $row);
        $row['validatedParamPlaneja'] = true;
      } else $paramPlan = $paramPlaneja;
      
      if(!\Zeedhi\Framework\Util\Functions::arrayKeyExists('IDGERARETI', $row) || $row['IDGERARETI'] == null){
        if(!empty($paramfilial)) {
          if($paramfilial['IDORDAJUINVE'] == null || $paramfilial['IDORDAJUINVE'] == 'P'){
            $row['ORDEM'] = '0';
            $dateFin->modify('-1 day');
            $row['DTINVENTRETI'] = $dateFin->format('d/m/Y');
          }else {
            // data inicial = data inicial + 1
            $row['DTINICIAL'] = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL'])->modify('+1 day')->format('d/m/Y');
            $row['ORDEM'] = 'U';
          }
          $row['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $nrorg);
          $this->procedureConsPlaPME($row);
          
          $row['IDGERARETI'] = $this->verifyEfet($row);
        }
      }
      
      // GRAVAR - itensNaoPlanejados inserir na lista
      
      /*Itens não planejados*/
      if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('unplannedItems', $row)) {
        $unplannedItems = $this->unplannedItems($row);
 
        if(!empty($unplannedItems) && $row['checkUnplannedProd']) {
          throw new \Exception("Produtos Não Planejados");
        } else {
          $serviceUnplanned = 'P'; //Padrão
        }
      } else {
        $serviceUnplanned = 'S'; //Selecionado
        $unplannedItems = $row['unplannedItems'];
      }
      
      $items = $this->itemEstoq($row);
      if (count($paramfilial) == 0) {
        $paramfilial = array("IDUTILLOTE"=>"N");
      }

      //ZERAR: rodar apenas uma vez, verificar se já foi validado produtos não planejados
      if($row['ZERAR']) $this->resetItems($row);
      
      $this->insertProdEstoq($items, $paramfilial);
      
      $this->procedureRetiradaMerenda($row);
      
      if ($serviceUnplanned === 'P') {
        if (count($unplannedItems) > 0) {
          // $paramCard = $this->validateCard($row, $paramPlan[0]['CDSERVRETPME'], $row['DTINVENTRETI']);
          // $this->insertUnplannedItemsLanc($unplannedItems, $row, $paramPlan, $paramCard, $paramfilial); 
          $this->insertUnplannedItemsLanc($unplannedItems, $row, $paramPlan, $paramPlan[0]['CDSERVRETPME'], $paramfilial); 
        }
      } else { // servico selecionado
        $unplannedItemsServ = array();
        foreach ($row['unplannedItems'] as $item) {
          if (!in_array($item['CDSERVFILI'], $unplannedItemsServ)) {
            $unplannedItemsServ[$item['CDSERVFILI']][] = $item;
          }
        }
        
        foreach ($unplannedItemsServ as $serv => $items) {
          // $paramCard = $this->validateCard($row, $serv, $row['DTINVENTRETI']);
          // $this->insertUnplannedItemsLanc($items, $row, $paramPlan, $paramCard, $paramfilial); 
          $this->insertUnplannedItemsLanc($items, $row, $paramPlan, $serv, $paramfilial); 
        }
      }
      
      // atualiza estoque procedure
      $this->recalcEstoqInterval($row, true, false);
      $irregularProducts = $this->estoqueService->getIrregularProducts();
      // gera lista dos produtos irregulares
      $nrlista = null;
      if(!empty($irregularProducts)) {
        // throw new \Exception("Não é possível realizar a retirada automática, pois existem produtos irregulares");
        $cdlista = '';
        foreach($irregularProducts as $item) {
          // if(is_array($item) && \Zeedhi\Framework\Util\Functions::arrayKeyExists("CDPRODUTO", $item)) $cdlista .= 'I'.$item['CDPRODUTO'].'_';
          // else $cdlista .= 'I'.$item.'_';
          if (is_string($item)) $cdlista .= 'I'.$item.'_';
          else $cdlista .= 'I'.$item->getCdProduto().'_';
        }
        // throw new \Exception('Produtos Irregulares');
        $nrlista = $this->insertLista($cdlista, $nrlista);
      }
      
      /*Trata a diferença do estoque depois do lançamento para criar ajuste*/
      $this->insertProdDiffEstoq($row, $paramPlan, $paramfilial);
      $this->insertLog($row);

      $this->updateStatus($row);
    
      if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('NRCONSPLAN', $row)) $this->execDeleteConsplan($row["NRCONSPLAN"], $row["CDFILCENTPME"]);
      
      // atualiza estoque apagando movilote
      $irregularProducts = $this->recalcEstoqInterval($row, true, false, true);
      // $this->recalcEstoqInterval($row);
      
      if (!isset($nrlista)) $nrlista = null;
      if(!empty($irregularProducts)) {
        // throw new \Exception("Não é possível realizar a retirada automática, pois existem produtos irregulares");
        $cdlista = '';
        foreach($irregularProducts as $item) {
          if(is_array($item) && \Zeedhi\Framework\Util\Functions::arrayKeyExists("CDPRODUTO", $item)) $cdlista .= 'I'.$item['CDPRODUTO'].'_';
          else $cdlista .= 'I'.$item.'_';
        }
        $nrlista = $this->insertLista($cdlista, $nrlista);
      }
      
      // gera lista dos produtos sem posicao de estoque
      $posStockProducts = $this->prodNotPosEstoq($row);
      if(!empty($posStockProducts)) {
        // throw new \Exception("Não é possível realizar a retirada automática, pois existem produtos que não possuem posição de estoque.");
        $cdlista = '';
        foreach($posStockProducts as $item) {
          $cdlista .= 'E'.$item['CDPRODUTO'].'_';
        }
        $nrlista = $this->insertLista($cdlista, $nrlista);      
      }
      
      // caso tenha produto nao planejado, retorna o serviço inserido
      $servRet = null;
      if(count($unplannedItems) > 0 && !empty($paramPlan[0])) {
        $servRet = array(
          'CDSERVRETPME' => $paramPlan[0]['CDSERVRETPME'],
          'NMSERVICO' => $paramPlan[0]['NMSERVICO'],
        );
      }
      
      $this->connection->commit('connection');
      $response->addDataSet(new DataSet('params', [$nrlista, $servRet]));
    } catch (\Exception $e) {
      if ($this->connection->isTransactionActive()) $this->connection->rollBack('connection');
      if($e->getMessage() == 'Não há lançamento de efetivos no período. Será realizado apenas o ajuste de inventário. Deseja continuar?'){
        $row['ERROR'] = 'Não há lançamento de efetivos no período. Será realizado apenas o ajuste de inventário. Deseja continuar?';
        $response->addDataSet(new DataSet('params', [$row]));
      } else if ($e->getMessage() == 'Produtos Não Planejados') {
        $row['ERROR'] = 'Produtos Não Planejados';
        $response->addDataSet(new DataSet('params', [$row]));
        // $response->setError(new Error($e->getMessage(), 500));
      } else if ($e->getMessage() == 'Produtos Irregulares') {
        $row['ERROR'] = 'Produtos Irregulares';
        $nrlista = $this->insertLista($cdlista, null);
        $response->addDataSet(new DataSet('params', [$nrlista, null]));
      } else $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  private function validateParamplaneja($paramPlan, $row){
    // $paramPlan = $this->getParamplaneja($row['CDFILIAL']);
    // if(empty($paramPlan) || $paramPlan[0]['CDSERVRETPME'] == null || $paramPlan[0]['IDSERVATIV'] == 'N') $this->execDeleteConsplan($row['NRCONSPLAN'], $row['CDFILIAL']);
    
    if(empty($paramPlan) || $paramPlan[0]['CDSERVRETPME'] == null) throw new \Exception('Operação Bloqueada! Serviço para retirada automática não cadastrada.');
    else if ($paramPlan[0]['IDSERVATIV'] == 'N')throw new \Exception('Serviço de Retirada Automática Inativo.');
    
    return $paramPlan;
  }

  private function recalcEstoqInterval($row, $verifyIrregularities, $abortAtIrregularities, $deleteMovilote = false) {
    $cardInterval = $this->getLancInterval($row);
    
    $irregularProducts = [];
    foreach($cardInterval as $card) {
      $irregularProductsCard = $this->callAtualizaEstoque($row['CDFILCENTPME'], $card['NRLANCESTQ'], [], $verifyIrregularities, $abortAtIrregularities,
      $deleteMovilote);
      $irregularProducts = array_merge($irregularProducts, $irregularProductsCard);
    }
    $irregularProducts = array_unique($irregularProducts);
    return $irregularProducts;
  }
  
  private function prodNotPosEstoq($row) {
    if($row['ZERAR']) {
      $query = "SELECT P.CDPRODUTO, P.NMPRODUTO, E.QTESTOQDIA FROM PRODUTO P
              LEFT JOIN POSESTDIA E ON E.CDPRODUTO = P.CDPRODUTO 
                AND E.DTPOSIESTQ = DTPOSICAO_POSESTDIA(E.CDFILIAL, P.CDPRODESTO, E.CDALMOXARIFE, E.CDLOCALESTOQ, E.NRLOTEESTQ, E.NRSUBLOTE, :P_DTPOSIESTQ)
             WHERE P.CDPRODUTO = P.CDPRODESTO AND P.IDPRODATIVPR = 'S' AND E.CDPRODUTO IS NULL
             ORDER BY P.NMPRODUTO";  
            // (SELECT MAX(E.DTPOSIESTQ)
            //   FROM POSESTDIA E
            //   WHERE E.CDFILIAL = :P_CDFILIAL  AND E.DTPOSIESTQ <= :P_DTPOSIESTQ AND P.CDPRODUTO = E.CDPRODUTO)
    } else {
      $query = "SELECT I.CDPRODUTO FROM ITEMINVENTRETI   I
                    LEFT JOIN POSESTDIA E ON E.CDPRODUTO = I.CDPRODUTO 
                    AND E.DTPOSIESTQ = DTPOSICAO_POSESTDIA(I.CDFILIAL, I.CDPRODUTO, E.CDALMOXARIFE, E.CDLOCALESTOQ, E.NRLOTEESTQ, E.NRSUBLOTE, :P_DTPOSIESTQ)
                WHERE I.CDFILIAL = :P_CDFILIAL AND I.NRINVENTRETI = :P_NRINVENTRETI AND E.CDPRODUTO IS NULL";
                // (SELECT MAX(E.DTPOSIESTQ) FROM POSESTDIA E
                //         WHERE E.CDFILIAL = I.CDFILIAL AND E.DTPOSIESTQ <= :P_DTPOSIESTQ AND E.CDPRODUTO = I.CDPRODUTO )
    }
    
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_DTPOSIESTQ' => $row['DTVALESTOQ'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
    );

    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function callAtualizaEstoque($cdfilial, $nrlancestq, $prods, $verifyIrregularities = true, $abortAtIrregularities = false, $deleteMovilote = false) {
    $irregularProductsCard = [];
    try{ 
      $filter = array(
        'nrorg' => $this->environment->getNrOrg(),
        'nrlancestq' => $nrlancestq,
        'cdfilial' => $cdfilial
      );
      
      $lanctoestoq = $this->entityManager->getRepository('\Estoque\Entities\Lanctoestoq')->findOneBy($filter); 

      if($lanctoestoq) {
        $this->estoqueService->setLanctoestoq($lanctoestoq);
        // verifica se possui irregularidades, caso tenha retorna os produtos com erro
        $irregularProductsCard = $this->estoqueService->atualizaEstoque($cdfilial, $nrlancestq,
        false, $verifyIrregularities, $abortAtIrregularities); // consolidate, verifyIrregularities, abortAtIrregularities   
      }
      /*
      $this->estoqueService->verificaIrregularidades($lanctoestoq);
      $irregularProductsCard = $this->estoqueService->getIrregularProducts();
      */
    } catch (\Exception $e) {
      if ($abortAtIrregularities) throw $e;
      $irregularProductsCard = $prods;
    }
    
    if($deleteMovilote && !empty($irregularProductsCard)) $this->deleteMovilote($cdfilial, $nrlancestq);
    return $irregularProductsCard;
  }

  private function deleteMovilote($cdfilial, $nrlancestq) {
    $nrMovilote = $cdfilial.$nrlancestq;

    $query = "DELETE FROM MOVILOTE WHERE NRMOVILOTE = :NRMOVILOTE";
    $params = array(
      "NRMOVILOTE" => $nrMovilote
    );
    $this->connection->prepare($query)->execute($params);
  }
  
  private function geraListaProd($items) {
    $cdlista = '';
    foreach($items as $item) {
      $cdlista .= $item['CDPRODUTO'].'_';
    }
    $nrlista = $this->insertLista($cdlista, null);
    return $nrlista;
  }
  
  private function insertLista($rows, $nrlista) {
    $rows = explode("_", $rows);
    $nrorg = $this->environment->getNrOrg();
    if(empty($nrlista)) $nrlista = $this->newCode->getCodeByTableName('lista', $nrorg);
    $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";

    foreach ($rows as $row) {
      if(!empty($row)){
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $row,
                "NRORG" => $nrorg
            )
        );
      }
    }

    return $nrlista;
  }

  private function getProdInterval($nrconsplan, $nrlista) {
    $query = "SELECT C.CDFILICPLA, C.CDSERVCPLA, C.DTCONSPLAN, C.IDCONSPLAN, C.CDPRODCPLA, C.QTCONSPLAN, C.VRPECONSPLAN, C.QTCOMESTCPLA, T.QTTOTAL
              FROM CONSPLAN C
              INNER JOIN LISTA L ON L.NRLISTA = :NRLISTA AND C.CDPRODCPLA = L.CDLISTA
              INNER JOIN (
                  SELECT C.CDPRODCPLA, SUM(C.QTCONSPLAN) QTTOTAL FROM CONSPLAN C
                  WHERE C.NRCONSPLAN = :NRCONSPLAN
                  GROUP BY C.CDPRODCPLA
              ) T ON T.CDPRODCPLA = C.CDPRODCPLA
              WHERE C.NRCONSPLAN = :NRCONSPLAN
              ORDER BY DTCONSPLAN";
    $params = array (
      'NRCONSPLAN' => $nrconsplan,
      'NRLISTA' => $nrlista
    );

    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function getQtTotalProds($nrconsplan, $nrlista) {
    $query = "SELECT C.CDPRODCPLA, SUM(C.QTCONSPLAN) QTTOTAL FROM CONSPLAN C
              INNER JOIN LISTA L ON L.NRLISTA = :NRLISTA AND C.CDPRODCPLA = L.CDLISTA
              WHERE C.NRCONSPLAN = :NRCONSPLAN
              GROUP BY C.CDPRODCPLA";
    $params = array (
      'NRCONSPLAN' => $nrconsplan,
      'NRLISTA' => $nrlista
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function insertProdDiffEstoq($row, &$paramPlan, $paramfilial) {
    /*Produtos com posição de estoque maior que o inventário*/
    $prodsQtDiff = $this->prodsQtDiff($row, $paramfilial);
    $nrlistaProd = $this->geraListaProd($prodsQtDiff);
    
    /*Planejamento no periodo desses produtos*/
    $prodsInterval = $this->getProdInterval($row['NRCONSPLAN'], $nrlistaProd);
    if(!empty($prodsInterval)) {
      $dtConsplan = '';
      $service = '';
      
      foreach($prodsInterval as $prod) {
        if($dtConsplan != $prod['DTCONSPLAN'] || $service != $prod['CDSERVCPLA']) {
          $paramCard = $this->validateCard($row, $prod['CDSERVCPLA'], $prod['DTCONSPLAN']);
          $nrSeqItem = $paramCard['NRSEQUITEM'];
          $dtConsplan = $prod['DTCONSPLAN'];
          $service = $prod['CDSERVCPLA'];
        }
        
        $qtDiff = 0;
        $vrFatorConv = 0;
        $vrCusto = 0;

        foreach($prodsQtDiff as $prodQtDiff) {
          if($prodQtDiff['CDPRODUTO'] === $prod['CDPRODCPLA']) {
            $qtDiff = $prodQtDiff['QTESTOQDIA'] - $prodQtDiff['QTITINVENTRETI'];
            $vrFatorConv = $prodQtDiff['VRFATOCONV'];
            $vrCusto = $prodQtDiff['VRCUSTPROD'];
            break;
          }
        }

        $percent = $prod['QTCONSPLAN'] / $prod['QTTOTAL'];
        
        $prod['CDFILIAL'] = $row['CDFILCENTPME'];
        $prod['CDPRODUTO'] = $prod['CDPRODCPLA'];
        $prod['CDPRODMOVI'] = $prod['CDPRODCPLA'];
        $prod['NRLANCESTQ'] = $paramCard['NRLANCESTQ'];
        $prod['QTLANCTOEST'] = round($percent * $qtDiff, 3);
        $prod['QTTOTLANCTO'] = $prod['QTLANCTOEST'];
        $prod['NRSEQUITEM'] = str_pad($nrSeqItem, 4, "0", STR_PAD_LEFT);
        $prod['CDFILIMOVI'] = $row['CDFILIAL'];
        $prod['CDALMOXARIFE'] = ' ';
        $prod['CDLOCALESTOQ'] = ' ';
        $prod['NRLOTEESTQ'] = ' ';
        $prod['NRSUBLOTE'] = ' ';
        $prod['IDTIPOMOVI'] = 3;
        $prod['IDEVOLESTOQ'] = 'S';
        $prod['DTLANCMOVI'] = $dtConsplan;
        $prod['IDORDEMCLC'] = $row['ORDEM'];
        $prod['CDTIPORETI'] = $paramPlan[0]['CDTPRETIPADR'];
        $prod['VRUNILANCTO'] = $vrCusto;
        $this->insertItemLancto($prod);


        $prod['CDFILIAL'] = $row['CDFILIAL'];
        $prod['CDFIITLANEST'] = $row['CDFILCENTPME'];
        $prod['QTRETIRADA'] = round($percent * $qtDiff, 3);
        $prod['NRRETIRADA'] = $paramCard['NRRETIRADA'];
        $prod['NRCARDAPIO'] = $paramCard['NRCARDAPIO'];
        $prod['NRSEQUITRETI'] = str_pad($paramCard['NRSEQUITEM'], 4, "0", STR_PAD_LEFT);
        
        $this->insertItemRetirada($prod);

        $paramCard['NRSEQUITEM'] += 1;
        $nrSeqItem += 1;
      }  
      $this->callAtualizaEstoque($row['CDFILCENTPME'], $paramCard['NRLANCESTQ'], $prodsQtDiff, false);
      
    }
    
    /* lança itens do inventário no lançamento de ajuste */
    $nrlanc = $this->insertLanctoEst($row["CDFILCENTPME"], $row['CDFILIAL'], '1', $row['DATAINVENT'], 'Retirada Automatica (Ajuste)');
    // $items = $this->getItemsInvent($row);
    $items = $row['itemsRomaneio'];
    $seq = 0;
    foreach($items as &$item) {
      $pos = $this->findPosProd($item, $row, $paramfilial);
      if(!empty($pos)) {
        if($pos['VRFATOCONV'] == '0') $qtLancEst = $item['QTITINVENTRETI'];
        else $qtLancEst = $pos['VRFATOCONV'] * $item['QTITINVENTRETI'];
  
        $item['CDFILIAL'] = $row['CDFILCENTPME'];
        $item['CDPRODMOVI'] = $item['CDPRODUTO'];
        $item['NRLANCESTQ'] = $nrlanc;
        $item['QTLANCTOEST'] = $qtLancEst;
        $item['NRSEQUITEM'] = str_pad($seq, 4, "0", STR_PAD_LEFT);
        $item['CDFILIMOVI'] = $row['CDFILIAL'];
        $item['CDALMOXARIFE'] = ' ';
        $item['CDLOCALESTOQ'] = ' ';
        $item['NRLOTEESTQ'] = ' ';
        $item['NRSUBLOTE'] = ' ';
        $item['IDTIPOMOVI'] = 1;
        $item['IDEVOLESTOQ'] = 'S';
        $item['DTLANCMOVI'] = $row['DATAINVENT'];
        $item['IDORDEMCLC'] = $row['ORDEM'];
        $item['CDTIPORETI'] = null;
        $item['VRUNILANCTO'] = $pos['VRCUSTPROD'];
        $item['QTTOTLANCTO'] = $qtLancEst;
        $this->insertItemLancto($item);
        $seq++; 
      }
    }
    $this->callAtualizaEstoque($row['CDFILCENTPME'], $nrlanc, $items, false);
  }

  private function getItemsInvent($row) {
    $query = "SELECT INV.CDFILIAL, INV.NRINVENTRETI, INV.DTINVENTRETI, IT.NRITINVENTRETI, IT.CDPRODUTO, PD.NMPRODUTO, 
                  PD.CDARVPROD, IT.QTITINVENTRETI, PD.SGUNIDADE, 'S' AS IDSERVICO, ' ' AS IDESTOQUE, IT.IDORIGLANCPRINV
              FROM INVENTARIORETI INV, ITEMINVENTRETI IT, PRODUTO PD
              WHERE INV.CDFILIAL = :P_CDFILIAL AND INV.NRINVENTRETI = :P_NRINVENTRETI AND INV.CDFILIAL = IT.CDFILIAL
              AND INV.NRINVENTRETI = IT.NRINVENTRETI AND PD.CDPRODUTO = IT.CDPRODUTO";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI']
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function prodsQtDiff($row, $paramfilial) {
    $query = "SELECT PR.CDPRODUTO, PR.QTITINVENTRETI, PR.VRFATOCONV, PR.QTESTOQDIA, PR.CDPRODESTO, CUSTO_PRODUTO(:P_CDFILIAL, PR.CDPRODUTO, :P_CDTIPCUSTEST, :P_DTPOSICAO, :P_DTATUAL) AS VRCUSTPROD
              FROM (
              SELECT P.CDPRODUTO, (IT.QTITINVENTRETI * P.VRFATOCONV) AS QTITINVENTRETI, P.VRFATOCONV, (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA,
                    P.CDPRODESTO
                FROM PRODUTO P, POSESTDIA D, PRODFILI F, INVENTARIORETI INV, ITEMINVENTRETI IT
              WHERE (INV.CDFILIAL = :P_CDFILIAL) AND (INV.NRINVENTRETI = :P_NRINVENTRETI) AND (INV.CDFILIAL = IT.CDFILIAL) AND (INV.NRINVENTRETI = IT.NRINVENTRETI)
                AND (P.CDPRODUTO = IT.CDPRODUTO) AND (D.CDFILIAL = :P_CDFILIAL) AND (D.CDALMOXARIFE = :P_CDALMOXARIFE) AND (D.CDLOCALESTOQ = :P_CDLOCALESTOQ)
                AND (D.NRLOTEESTQ = :P_NRLOTEESTQ) AND (D.NRSUBLOTE = :P_NRSUBLOTE) AND (D.CDPRODUTO = P.CDPRODESTO) AND (P.CDPRODESTO = F.CDPRODUTO)
                AND (F.CDFILIAL = :P_CDFILIAL)
                AND (DTPOSIESTQ = DTPOSICAO_POSESTDIA(:P_CDFILIAL, P.CDPRODESTO, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_DTPOSICAO))
              ) PR
              WHERE (PR.QTESTOQDIA - (PR.QTESTOQDIA - PR.QTITINVENTRETI) > PR.QTITINVENTRETI)";
              
              // , NVL(PF.CDTIPCUSTEST, 3),
              // PARAMFILIAL PF, 
              // AND (PF.CDFILIAL = F.CDFILIAL)
    $params = array (
      "P_CDFILIAL" => $row['CDFILIAL'],
      "P_CDALMOXARIFE" => ' ',
      "P_CDLOCALESTOQ" => ' ',
      "P_NRLOTEESTQ" => ' ',
      "P_NRSUBLOTE" => ' ',
      "P_NRINVENTRETI" => $row['NRINVENTRETI'],
      "P_DTPOSICAO" => $row['DTVALESTOQ'],
      "P_DTATUAL" => $row['DTPROCESSA'],
      "P_CDTIPCUSTEST" => $paramfilial['CDTIPCUSTEST'],
    );
    $data = $this->connection->fetchAll($query, $params);
    
    return $data;
  }

  private function updateStatus($row) {
    $update = "UPDATE INVENTARIORETI SET IDRETIEFET = 'S' WHERE CDFILIAL = :P_CDFILIAL AND NRINVENTRETI = :P_NRINVENTRETI";
    $params = array(
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
    );
    $stmt = $this->connection->prepare($update);   
    $stmt->execute($params);
  }

  private function insertLog($row) {
    $nrseq = $this->newCode->getCodeByTableName('LOGINVENTARIORETI', $this->environment->getNrOrg());

    $cdoperador = $this->environment->getCdOperador();
    $opcao = $row['ZERAR'] ? 11 : 10;

    $update = "INSERT INTO LOGINVENTARIORETI (NRLOGINVENTARIORETI, CDFILIAL, NRINVENTRETI, DTINCLUSAO, DTINICIALINVENTARIO, DTFINALINVENTARIO,
                  CDOPERINCLUSAO, IDOPCAOGRAVAPRODUTOS, IDRETIRADAGERADA)
              VALUES (:P_NRLOGINVENTARIORETI, :P_CDFILIAL, :P_NRINVENTRETI, SYSDATE, :P_DTINICIALINVENTARIO, :P_DTFINALINVENTARIO,
                :P_CDOPERINCLUSAO, DECODE (:P_IDOPCAOGRAVAPRODUTOS, 10, 'GRAVA_INFORMADOS', DECODE (:P_IDOPCAOGRAVAPRODUTOS, 11, 'ZERA_NAO_INFORMADOS')), :P_IDRETIRADAGERADA)";
    $params = array(
      'P_NRLOGINVENTARIORETI' => $nrseq,
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
      'P_DTINICIALINVENTARIO' => $row['DTINICIAL'],
      'P_DTFINALINVENTARIO' => $row['DATAINVENT'],
      'P_CDOPERINCLUSAO' => $cdoperador,
      'P_IDOPCAOGRAVAPRODUTOS' => $opcao,
      'P_IDRETIRADAGERADA' => $row['IDGERARETI'],
    );

    $stmt = $this->connection->prepare($update);   
    $stmt->execute($params);

  }

  private function procedureConsPlaPME($row) {
    $cdoperador = $this->environment->getCdOperador();
    $procedureCons = new StoredProcedure($this->connection, 'CONSUMO_PLANEJADO_PME');

    $procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $row['CDFILCENTPME'], Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, null,                 Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL'],     Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, null,                 Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],    Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTINVENTRETI'], Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, 'C',                  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, 'CFP',                Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, '0',                  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, 'N',                  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $row['NRCONSPLAN'],   Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                  Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'ZZZZZZZZZ',          Param::PARAM_TYPE_STR));
    $procedureCons->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador,          Param::PARAM_TYPE_STR));
    $procedureCons->execute();
  }

  private function procedureRetiradaMerenda($row) {
    $cdoperador = $this->environment->getCdOperador();

    $procedureRet = new StoredProcedure($this->connection, 'GERA_RETIRADA_MERENDA');

    $procedureRet->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL'],     Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $row['CDFILCENTPME'], Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],    Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DATAINVENT'],   Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $row['NRCONSPLAN'],   Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador,          Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_NRINVENTRETI', Param::PARAM_INPUT, $row['NRINVENTRETI'], Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $row['CDFILCENTPME'], Param::PARAM_TYPE_STR));
    $procedureRet->addParam(new Param('P_DSLANCAMENTO', Param::PARAM_INPUT, $row['DSINVENTRETI'], Param::PARAM_TYPE_STR));
    $procedureRet->execute();
  }

  private function getParamFilial($row) {
    $query = "SELECT IDORDAJUINVE, IDUTILLOTE, CDALMOXARIFE, CDLOCALESTOQ, IDUTILALMOX, NVL(CDTIPCUSTEST, 3) CDTIPCUSTEST
                FROM PARAMFILIAL 
              WHERE CDFILIAL  = :P_CDFILIAL";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL']
    );
    $data = $this->connection->fetchAll($query, $params);
    
    if(!empty($data))$result = $data[0];
    else $result = [];

    return $result;
  }

  private function verifyEfet($row) {
    $query = "SELECT NVL(SUM(QTCOMEFECPME), 0) AS QTCOMEFECPME
                FROM CARDIAPME CAR
              WHERE CDFILCENTPME = :P_CDFILCENTPME
                AND CDFILIAL = :P_CDFILIAL
                AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL
                AND EXISTS (SELECT CDSERVCPLA
                              FROM CONSPLAN CON
                              WHERE CON.NRCONSPLAN = :P_NRCONSPLAN
                                AND CON.CDFILICPLA = CAR.CDFILIAL
                                AND CON.DTCONSPLAN = CAR.DTCARDIAPME)";
    $params = array (
      'P_CDFILCENTPME' => $row['CDFILCENTPME'],
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_DTINICIAL' => $row['DTINICIAL'],
      'P_DTFINAL' => $row['DATAINVENT'],
      'P_NRCONSPLAN' => $row['NRCONSPLAN']
    );
    
    $data = $this->connection->fetchAll($query, $params);
    
    if(empty($data) || $data[0]['QTCOMEFECPME'] == 0) throw new \Exception('Não há lançamento de efetivos no período. Será realizado apenas o ajuste de inventário. Deseja continuar?');

    return 'S';
  }

  public function deleteConsplan(DTO\Request $request, DTO\Response $response){
    try{
      $row = $request->getParameters();
      $this->execDeleteConsplan($row["NRCONSPLAN"], $row["CDFILCENTPME"]);
    }catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function execDeleteConsplan($nrconsplan, $unidade) {
    $query = "DELETE FROM CONSPLAN WHERE  (NRCONSPLAN = :P_NRCONSPLAN) AND (CDFILCENTPME = :P_CDFILCENTPME)";
    $params = array(
      "P_NRCONSPLAN" => $nrconsplan,
      "P_CDFILCENTPME" => $unidade
    );
    $this->connection->prepare($query)->execute($params);
  }

  private function unplannedItems($row) {
    $query = "SELECT INV.CDFILIAL, INV.NRINVENTRETI, INV.DTINVENTRETI, IT.NRITINVENTRETI, PD.CDPRODESTO, PD.VRFATOCONV,
                  IT.CDPRODUTO, PD.NMPRODUTO, IT.QTITINVENTRETI, PD.SGUNIDADE, 'N' AS IDSERVICO
              FROM INVENTARIORETI INV, ITEMINVENTRETI IT, PRODUTO PD
              WHERE INV.CDFILIAL = :P_CDFILIAL
              AND INV.NRINVENTRETI = :P_NRINVENTRETI
              AND INV.CDFILIAL = IT.CDFILIAL
              AND INV.NRINVENTRETI = IT.NRINVENTRETI
              AND PD.CDPRODUTO = IT.CDPRODUTO
              AND NOT EXISTS (SELECT CP.CDPRODCPLA
                                FROM CONSPLAN CP
                               WHERE CP.NRCONSPLAN = :P_NRCONSPLAN
                                 AND CP.CDFILICPLA = :P_CDFILIAL
                                 AND CP.CDPRODCPLA = IT.CDPRODUTO)";
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
      'P_NRCONSPLAN' => $row['NRCONSPLAN']
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function itemEstoq($row) {
    $query = "SELECT  I.CDFILIAL AS CDFILIMOVI, I.NRINVENTRETI, I.CDPRODUTO AS CDPRODMOVI,
                  ' ' AS CDALMOXARIFE, ' ' AS CDLOCALESTOQ, ' ' AS NRLOTEESTQ, ' ' AS NRSUBLOTE, NVL(PF.IDUTCTRLLOTE, 'N') IDUTCTRLLOTE
              FROM ITEMINVENTRETI I
             
              LEFT JOIN PRODFILI PF ON (PF.CDFILIAL  = I.CDFILIAL)  AND (PF.CDPRODUTO  = I.CDPRODUTO)
              WHERE I.CDFILIAL = :P_CDFILIAL AND I.NRINVENTRETI = :P_NRINVENTRETI";
              // , NVL(F.IDUTILLOTE, 'N') IDUTILLOTE
              // INNER JOIN PARAMFILIAL F ON F.CDFILIAL = I.CDFILIAL
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI']
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }

  private function resetItems($row) {
    $query = "SELECT P.CDPRODUTO
                FROM POSESTDIA P, PRODUTO PR
              WHERE P.CDFILIAL = :P_CDFILIAL
              AND P.CDPRODUTO = PR.CDPRODUTO
                AND PR.CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO)
                AND P.DTPOSIESTQ = DTPOSICAO_POSESTDIA(:P_CDFILIAL, P.CDPRODUTO, P.CDALMOXARIFE, P.CDLOCALESTOQ, P.NRLOTEESTQ, P.NRSUBLOTE, :P_DTPOSIESTQ)
                
              MINUS
              SELECT  I.CDPRODUTO
                FROM ITEMINVENTRETI I
              WHERE I.CDFILIAL = :P_CDFILIAL AND I.NRINVENTRETI = :P_NRINVENTRETI";
              // (SELECT MAX(E.DTPOSIESTQ)
              //                         FROM POSESTDIA E
              //                         WHERE E.CDFILIAL = :P_CDFILIAL AND E.DTPOSIESTQ <= :P_DTPOSIESTQ AND P.CDPRODUTO = E.CDPRODUTO)
    $params = array (
      'P_CDFILIAL' => $row['CDFILIAL'],
      'P_NRINVENTRETI' => $row['NRINVENTRETI'],
      'P_DTPOSIESTQ' => $row['DTVALESTOQ']
    );
    $data = $this->connection->fetchAll($query, $params);
    
    if ($data) {

      $nrSeqItem = $this->getMaxSeqItem($row['CDFILIAL'], $row['NRINVENTRETI']);
  
      $queryInsert = "INSERT INTO ITEMINVENTRETI (CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV)
                      VALUES (:CDFILIAL, :NRINVENTRETI, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV)";
  
      foreach($data as $item) {
        $nrSeqItem += 1;
        $params = array (
          'CDFILIAL' => $row['CDFILIAL'],
          'NRINVENTRETI' => $row['NRINVENTRETI'],
          'NRITINVENTRETI' => $nrSeqItem,
          'QTITINVENTRETI' => 0,
          'CDPRODUTO' => $item['CDPRODUTO'],
          'IDORIGLANCPRINV' => 0
        );
        $this->connection->prepare($queryInsert)->execute($params);
      }
    }
  }

  private function insertProdEstoq($items, $paramfilial) {
    foreach($items as  $item) {
      $query = "INSERT INTO PRODLCESTOQ
                (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO)
                (SELECT :P_CDFILIAL, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_CDPRODUTO
                FROM PRODFILI
                WHERE CDFILIAL  = :P_CDFILIAL  AND
                      CDPRODUTO = :P_CDPRODUTO AND
                      NVL(IDCNTRESTOQ, 'S') = 'S'
                MINUS
                SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
                FROM PRODLCESTOQ
                WHERE CDFILIAL     = :P_CDFILIAL     AND
                      CDALMOXARIFE = :P_CDALMOXARIFE AND
                      CDLOCALESTOQ = :P_CDLOCALESTOQ AND
                      CDPRODUTO    = :P_CDPRODUTO)";

      $params = array(
        'P_CDFILIAL' => $item['CDFILIMOVI'],
        'P_CDALMOXARIFE' => ' ',
        'P_CDLOCALESTOQ' => ' ',
        'P_CDPRODUTO' => $item['CDPRODMOVI']
      );
      $stmt = $this->connection->prepare($query);   
      $stmt->execute($params);

      // LOTE
      // && $item['IDUTILLOTE'] == 'S'
      if($paramfilial['IDUTILLOTE'] == 'S') {
        $lote = $this->findLote($item, $paramfilial);
        if (empty($lote)) {
          $item['NRLOTEESTQ'] = ' ';
          $item['NRSUBLOTE'] = ' ';
        } else {
          $item['NRLOTEESTQ'] = $lote['NRLOTEESTQ'];
          $item['NRSUBLOTE'] = $lote['NRSUBLOTE'];
        }
      }else {
        $item['NRLOTEESTQ'] = ' ';
        $item['NRSUBLOTE'] = ' ';
      }

      if($this->loteIsValid($item)) {
        $query = "INSERT INTO LOTE(CDFILIAL, NRLOTEESTQ, NRSUBLOTE, CDPRODUTO, IDSTATLOTE, DTENTRLOTE)
        (SELECT :P_CDFILIAL CDFILIAL, :P_NRLOTEESTQ NRLOTEESTQ, :P_NRSUBLOTE NRSUBLOTE, :P_CDPRODUTO CDPRODUTO,  'L' IDSTATLOTE, SYSDATE DTENTRLOTE
          FROM PARAMGERAL
          MINUS
          SELECT CDFILIAL, NRLOTEESTQ, NRSUBLOTE, :P_CDPRODUTO CDPRODUTO, 'L', SYSDATE
          FROM LOTE
          WHERE :P_CDFILIAL = CDFILIAL AND :P_NRLOTEESTQ = NRLOTEESTQ AND :P_NRSUBLOTE = NRSUBLOTE AND :P_CDPRODUTO = CDPRODUTO)";

        $params = array(
          "P_CDFILIAL" => $item['CDFILIMOVI'],
          "P_NRLOTEESTQ" => $item['NRLOTEESTQ'],
          "P_NRSUBLOTE" => $item['NRSUBLOTE'],
          "P_CDPRODUTO" => $item['CDPRODMOVI'],
        );
        $stmt = $this->connection->prepare($query);   
        $stmt->execute($params);
      }
    }
  }

  private function findLote($item, $paramfilial) {
    $query = "SELECT NRLOTEESTQ, NRSUBLOTE, QTTOTALESTOQ, VRTOTALESTOQ
              FROM POSESTATU
              WHERE CDFILIAL     = :P_CDFILIAL  AND CDALMOXARIFE = :P_CDALMOXARIFE AND CDLOCALESTOQ = :P_CDLOCALESTOQ AND
                    CDPRODUTO    = :P_CDPRODUTO AND QTTOTALESTOQ > 0
              ORDER BY NRLOTEESTQ, NRSUBLOTE";
    $params = array (
      "P_CDFILIAL" => $item['CDFILIMOVI'],
      "P_CDALMOXARIFE" => $paramfilial['CDALMOXARIFE'],
      "P_CDLOCALESTOQ" => $paramfilial['CDLOCALESTOQ'],
      "P_CDPRODUTO" => $item['CDPRODMOVI'],
    );
    $data = $this->connection->fetchAssoc($query, $params);
    return $data;
  }

  private function loteIsValid($item) {
    $result = false;
    $query = "SELECT CDFILIAL
              FROM LOTE
              WHERE (:P_CDFILIAL  = CDFILIAL)  AND (:P_NRLOTEESTQ = NRLOTEESTQ) AND
                    (:P_NRSUBLOTE = NRSUBLOTE) AND (:P_CDPRODUTO  = CDPRODUTO)";
    $params = array (
      "P_CDFILIAL" => $item['CDFILIMOVI'],
      "P_NRLOTEESTQ" => $item['NRLOTEESTQ'],
      "P_NRSUBLOTE" => $item['NRSUBLOTE'],
      "P_CDPRODUTO" => $item['CDPRODMOVI'],
    );
    $data = $this->connection->fetchAll($query, $params);
    if(empty($data)) $result = true;
    
    return $result;
  }

  private function insertUnplannedItemsLanc(&$items, $row, &$paramPlan, $serv, $paramfilial) {
    $nrorg = $this->environment->getNrorg();
    foreach($items as &$item) {
      $pos = $this->findPosProd($item, $row, $paramfilial);
      if (!empty($pos)) {
        if($pos['VRFATOCONV'] == '0') $qtLancEst = $pos['QTESTOQDIA'] - $item['QTITINVENTRETI'];
        else $qtLancEst = $pos['QTESTOQDIA'] - ($pos['VRFATOCONV'] * $item['QTITINVENTRETI']);
  
        if($qtLancEst < 0) {
          continue;
          // $qtLancEst = $pos['QTESTOQDIA'] * (-1);
        }
        $paramCard = $this->validateCard($row, $serv, $row['DTINVENTRETI']);
        $idtipomovi = 3;
        /*
        if($qtLancEst < 0){
          $qtLancEst = $pos['QTESTOQDIA'] * (-1);
          $idtipomovi = 1;
        } 
        */
        
        $item['CDFILIAL'] = $row['CDFILCENTPME'];
        $item['CDPRODMOVI'] = $item['CDPRODUTO'];
        $item['NRLANCESTQ'] = $paramCard['NRLANCESTQ'];
        $item['QTLANCTOEST'] = $qtLancEst;
        $item['NRSEQUITEM'] = str_pad($paramCard['NRSEQUITEM'], 4, "0", STR_PAD_LEFT);
        $item['CDFILIMOVI'] = $row['CDFILIAL'];
        $item['CDALMOXARIFE'] = ' ';
        $item['CDLOCALESTOQ'] = ' ';
        $item['NRLOTEESTQ'] = ' ';
        $item['NRSUBLOTE'] = ' ';
        $item['IDTIPOMOVI'] = $idtipomovi;
        $item['IDEVOLESTOQ'] = 'S';
        // $item['DTLANCMOVI'] = $row['DTINVENTRETI'];
        $item['DTLANCMOVI'] = $row['DTINVENTRETI'];
        $item['IDORDEMCLC'] = $row['ORDEM'];
        $item['CDTIPORETI'] = $paramPlan[0]['CDTPRETIPADR'];
        $item['VRUNILANCTO'] = $pos['VRCUSTPROD'];
        $item['QTTOTLANCTO'] = $item['QTITINVENTRETI'];
        
        $this->insertItemLancto($item);
        
        // if($idtipomovi == 3) {
          $item['CDFILIAL'] = $row['CDFILIAL'];
          $item['CDFIITLANEST'] = $row['CDFILCENTPME'];
          $item['QTRETIRADA'] = round($qtLancEst, 3);
          $item['NRRETIRADA'] = $paramCard['NRRETIRADA'];
          $item['NRCARDAPIO'] = $paramCard['NRCARDAPIO'];
          $item['NRSEQUITRETI'] = str_pad($paramCard['NRSEQUITEM'], 4, "0", STR_PAD_LEFT);
          
          $this->insertItemRetirada($item); 
        // }
  
        $paramCard['NRSEQUITEM'] += 1;
      }
    }
    // $this->callAtualizaEstoque($row['CDFILCENTPME'], $paramCard['NRLANCESTQ'], [], false);
  }

  private function insertLanctoEst($cdfilial, $cdfilimovi, $idtplanc, $dtlanc, $dslanc) {
    $nrseq = $this->newCode->getCodeByTableName('LANCTOEST'.$cdfilial, $this->environment->getNrOrg());
    $insert = "INSERT INTO LANCTOESTOQ (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, CDFILIMOVI, IDALTCUSMED, CDOPERADOR, DSLANCESTQ)
              VALUES (:P_CDFILIAL, :P_NRLANCESTQ, :P_IDTPLANCTO, :P_DTLANCESTQ, :P_CDFILIMOVI, :P_IDALTCUSMED, :P_CDOPERADOR, :P_DSLANCESTQ)";
    $params = array(
      'P_CDFILIAL' => $cdfilial,
      'P_NRLANCESTQ' => $nrseq,
      'P_IDTPLANCTO' => $idtplanc,
      'P_DTLANCESTQ' => $dtlanc,
      'P_CDFILIMOVI' => $cdfilimovi,
      'P_IDALTCUSMED'=> 'N',
      'P_CDOPERADOR' => $this->environment->getCdOperador(),
      'P_DSLANCESTQ' => $dslanc,
    );
    $stmt = $this->connection->prepare($insert);   
    $stmt->execute($params);
    return $nrseq;
  }

  private function insertItemLancto($item) {
    $insert = "INSERT INTO ITLANCTOEST (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, VRUNILANCTO, QTTOTLANCTO, CDPRODMOVI, CDFILIMOVI, 
                    CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, IDTIPOMOVI, IDEVOLESTOQ, DTLANCMOVI, IDORDEMCLC, CDTIPORETI)
               VALUES (:P_CDFILIAL, :P_NRLANCESTQ, :P_NRSEQUITEM, :P_CDPRODUTO, :P_QTLANCTOEST, :P_VRUNILANCTO, :P_QTTOTLANCTO, :P_CDPRODMOVI, :P_CDFILIMOVI,
                    :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_IDTIPOMOVI, :P_IDEVOLESTOQ, :P_DTLANCMOVI, :P_IDORDEMCLC, :P_CDTIPORETI)";
    
    $params = array(
      'P_CDFILIAL' => $item['CDFILIAL'],
      'P_NRLANCESTQ' => $item['NRLANCESTQ'],
      'P_NRSEQUITEM' => $item['NRSEQUITEM'],
      'P_CDPRODUTO' => $item['CDPRODUTO'],
      'P_QTLANCTOEST' => $item['QTLANCTOEST'],
      'P_VRUNILANCTO' => $item['VRUNILANCTO'],
      'P_QTTOTLANCTO' => $item['QTTOTLANCTO'],
      'P_CDPRODMOVI' => $item['CDPRODMOVI'],
      'P_CDFILIMOVI' => $item['CDFILIMOVI'],
      'P_CDALMOXARIFE' => $item['CDALMOXARIFE'],
      'P_CDLOCALESTOQ' => $item['CDLOCALESTOQ'],
      'P_NRLOTEESTQ' => $item['NRLOTEESTQ'],
      'P_NRSUBLOTE' => $item['NRSUBLOTE'],
      'P_IDTIPOMOVI' => $item['IDTIPOMOVI'],
      'P_IDEVOLESTOQ' => $item['IDEVOLESTOQ'],
      'P_DTLANCMOVI' => $item['DTLANCMOVI'],
      'P_IDORDEMCLC' => $item['IDORDEMCLC'],
      'P_CDTIPORETI' => $item['CDTIPORETI'],
    );
    $stmt = $this->connection->prepare($insert);   
    $stmt->execute($params);
  }

  private function insertRetirada($cdfilial, $nrcard, $dtret, $cdfilanc, $nrlanc, $nrinvent) {
    $nrseq = $this->newCode->getCodeByTableName('RETIRADA'.$cdfilial, $this->environment->getNrOrg());
    $insert = "INSERT INTO RETIRADA (CDFILIAL, NRRETIRADA, NRCARDAPIO, DTRETIRADA, CDFILANCESTQ, NRLANCESTQ, IDRETIPRAT, NRINVENTRETI)
              VALUES (:P_CDFILIAL, :P_NRRETIRADA, :P_NRCARDAPIO, :P_DTRETIRADA, :P_CDFILANCESTQ, :P_NRLANCESTQ, :P_IDRETIPRAT, :P_NRINVENTRETI)";
    $params = array(
      'P_CDFILIAL' => $cdfilial,
      'P_NRRETIRADA' => $nrseq,
      'P_NRCARDAPIO' => $nrcard,
      'P_DTRETIRADA' => $dtret,
      'P_CDFILANCESTQ' => $cdfilanc,
      'P_NRLANCESTQ'=> $nrlanc,
      'P_IDRETIPRAT' => 'N',
      'P_NRINVENTRETI' => $nrinvent,
    );
    $stmt = $this->connection->prepare($insert);   
    $stmt->execute($params);
    return $nrseq;
  }

  private function insertItemRetirada($item) {
    $insert = "INSERT INTO ITEMRETI (CDFILIAL, NRRETIRADA, NRCARDAPIO, NRSEQUITRETI, CDPRODUTO, QTRETIRADA, CDFIITLANEST, NRLANCESTQ, NRSEQUITEM, CDTIPORETI)
              VALUES (:P_CDFILIAL, :P_NRRETIRADA, :P_NRCARDAPIO, :P_NRSEQUITRETI, :P_CDPRODUTO, :P_QTRETIRADA, :P_CDFIITLANEST, :P_NRLANCESTQ, :P_NRSEQUITEM, :P_CDTIPORETI)";
    
    $params = array(
      'P_CDFILIAL' => $item['CDFILIAL'],
      'P_NRRETIRADA' => $item['NRRETIRADA'],
      'P_NRCARDAPIO' => $item['NRCARDAPIO'],
      'P_NRSEQUITRETI' => $item['NRSEQUITRETI'],
      'P_CDPRODUTO' => $item['CDPRODUTO'],
      'P_QTRETIRADA' => $item['QTRETIRADA'],
      'P_CDFIITLANEST' => $item['CDFIITLANEST'],
      'P_NRLANCESTQ' => $item['NRLANCESTQ'],
      'P_NRSEQUITEM' => $item['NRSEQUITEM'],
      'P_CDTIPORETI' => $item['CDTIPORETI'],
    );
    $stmt = $this->connection->prepare($insert);   
    $stmt->execute($params);
  }

  private function validateCard($row, $cdservfili, $date) {
    $result = [];
    $query = "SELECT CA.NRCARDAPIO, RE.NRRETIRADA, RE.CDFILANCESTQ, RE.NRLANCESTQ
              FROM CARDADIA CA
              LEFT JOIN RETIRADA RE ON RE.NRCARDAPIO = CA.NRCARDAPIO
                                   AND RE.CDFILIAL = CA.CDFILIAL
              WHERE CA.CDFILIAL = :P_CDFILIAL AND CA.CDSERVFILI = :P_CDSERVFILI 
                AND DTCARDAPIO = :P_DTCARDAPIO";
    $params = array (
      "P_CDFILIAL" => $row['CDFILIAL'],
      "P_CDSERVFILI" => $cdservfili,
      "P_DTCARDAPIO" => $date
    );

    $card = $this->connection->fetchAll($query, $params);
    if(count($card) === 0) { // insere caso não exista cardapio
      
      $nrcard = $this->newCode->getCodeByTableName('CARDAPIO'.$row['CDFILIAL'], $this->environment->getNrOrg());
      $dtmescard = '01'.substr($date, 2, strlen($date));

      $queryRomaneio = "INSERT INTO CARDADIA (CDFILIAL, NRCARDAPIO, CDSERVFILI, CDOPERADOR, DTCARDAPIO, DTMESCARD)
                        VALUES (:CDFILIMOVI, :NRCARDAPIO, :CDSERVFILI, :CDOPERADOR, :DTCARDAPIO, :DTMESCARD)";
      $params = array(
        'CDFILIMOVI' => $row['CDFILIAL'],
        'NRCARDAPIO' => $nrcard,
        'CDSERVFILI' => $cdservfili,
        'CDOPERADOR' => $this->environment->getCdOperador(),
        'DTCARDAPIO' => $date,
        'DTMESCARD'  => $dtmescard
      );
     
      $stmt = $this->connection->prepare($queryRomaneio);   
      $stmt->execute($params);
      $nrlanc = $this->insertLanctoEst($row["CDFILCENTPME"], $row['CDFILIAL'], '3', $date, 'Retirada Automatica (Itens não planejados)');
      $nrret = $this->insertRetirada($row['CDFILIAL'], $nrcard, $date, $row["CDFILCENTPME"], $nrlanc, $row['NRINVENTRETI']);
      
      $result['NRSEQUITEM'] = 1;
      $result['NRLANCESTQ'] = $nrlanc;
      $result['NRRETIRADA'] = $nrret;
      
      $result['NRCARDAPIO'] = $nrcard;
    } else {
      $result['NRCARDAPIO'] = $card[0]['NRCARDAPIO'];
      
      if (!empty($card[0]['NRRETIRADA'])) {
        $result['NRRETIRADA'] = $card[0]['NRRETIRADA'];
        $result['NRLANCESTQ'] = $card[0]['NRLANCESTQ'];
        
        $result['NRSEQUITEM'] = $this->getMaxSeqRet($row['CDFILIAL'], $result['NRCARDAPIO'], $card[0]['NRRETIRADA']);
      } else {
        $nrlanc = $this->insertLanctoEst($row["CDFILCENTPME"], $row['CDFILIAL'], '3', $date, 'Retirada Automatica (AQUI)');
        $nrret = $this->insertRetirada($row['CDFILIAL'], $result['NRCARDAPIO'], $date, $row["CDFILCENTPME"], $nrlanc, $row['NRINVENTRETI']);
        
        $result['NRSEQUITEM'] = 1;
        $result['NRLANCESTQ'] = $nrlanc;
        $result['NRRETIRADA'] = $nrret;
      }
    }
    
    return $result;
  }

  private function getMaxSeqRet($cdfilial, $nrcard, $nrretirada) {
    $query = "SELECT NVL(MAX(NRSEQUITRETI) + 1, 1) AS NRSEQUITEM FROM ITEMRETI
              WHERE CDFILIAL = :P_CDFILIAL AND NRCARDAPIO = :P_NRCARDAPIO AND NRRETIRADA = :P_NRRETIRADA";
    $params = array (
      "P_CDFILIAL" => $cdfilial,
      "P_NRCARDAPIO" => $nrcard,
      "P_NRRETIRADA" => $nrretirada
    );
    

    return $this->connection->fetchColumn($query, $params);
  }

  private function findPosProd($item, $row, $paramfilial) {
    $query = "SELECT P.CDPRODUTO, P.NMPRODUTO,  P.SGUNIDADE, IDCNTRESTOQ, P.VRFATOCONV, (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA,
                  CUSTO_PRODUTO(:P_CDFILIAL, :P_CDPRODUTO, :P_CDTIPCUSTEST, :P_DTPOSICAO, :P_DTATUAL) AS  VRCUSTPROD, P.CDPRODESTO, (QTESTOQANT/P.VRFATOCONV)  as QTESTOQANT
              FROM PRODUTO P, POSESTDIA D, PRODFILI F
              WHERE (P.CDPRODUTO = :P_CDPRODUTO) AND (D.CDFILIAL = :P_CDFILIAL) AND (D.CDALMOXARIFE = :P_CDALMOXARIFE) AND (D.CDLOCALESTOQ = :P_CDLOCALESTOQ)
              AND (D.NRLOTEESTQ = :P_NRLOTEESTQ) AND (D.NRSUBLOTE = :P_NRSUBLOTE) AND (D.CDPRODUTO = P.CDPRODESTO) AND (P.CDPRODESTO = F.CDPRODUTO)
              AND (F.CDFILIAL = :P_CDFILIAL)
              AND (DTPOSIESTQ = DTPOSICAO_POSESTDIA(D.CDFILIAL, P.CDPRODESTO, D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, :P_DTPOSICAO))";
              // AND (DTPOSIESTQ = (SELECT MAX(DTPOSIESTQ)
              //                     FROM POSESTDIA
              //                     WHERE (CDFILIAL = :P_CDFILIAL) AND (CDALMOXARIFE = :P_CDALMOXARIFE) AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
              //                       AND (NRLOTEESTQ = :P_NRLOTEESTQ) AND (NRSUBLOTE = :P_NRSUBLOTE) AND (CDPRODUTO = P.CDPRODESTO) AND (DTPOSIESTQ  <= :P_DTPOSICAO)))";
    $params = array (
      "P_CDFILIAL" => $row['CDFILIAL'],
      "P_CDALMOXARIFE" => ' ',
      "P_CDLOCALESTOQ" => ' ',
      "P_NRLOTEESTQ" => ' ',
      "P_NRSUBLOTE" => ' ',
      "P_CDPRODUTO" => $item['CDPRODUTO'],
      "P_DTPOSICAO" => $row['DTVALESTOQ'],
      "P_DTATUAL" => $row['DTPROCESSA'],
      "P_CDTIPCUSTEST" => $paramfilial['CDTIPCUSTEST'],
    );
    
    $data = $this->connection->fetchAll($query, $params);
    if (!empty($data)) return $data[0];
    else return null;
  }

  private function getLancInterval($row) {
    $query = "SELECT L.CDFILIAL, L.CDFILIMOVI, L.DTLANCESTQ, L.NRLANCESTQ, L.IDTPLANCTO
              FROM LANCTOESTOQ L
              WHERE L.CDFILIMOVI = :P_CDFILIAL AND L.DTLANCESTQ BETWEEN :P_DTINICIAL AND :P_DTFINAL";
             //ORDER BY L.CDFILIAL, L.CDFILIMOVI, L.DTLANCESTQ";
    $params = array (
      "P_CDFILIAL" => $row['CDFILIAL'],
      "P_DTINICIAL" => $row['DTINICIAL'],
      "P_DTFINAL" => $row['DATAINVENT'],
      //"P_DTFINAL" => $row['DTVALESTOQ'],
    );
    $data = $this->connection->fetchAll($query, $params);
    return $data;
  }
}